import { Service } from "./service"
import * as express from "express"

export interface Request extends express.Request {
}
export interface Response extends express.Response { }
interface RouteHandlerConstructor {
    new (service: RouteService): Route<any>
}
export abstract class RouteService extends Service {
    public routes: Route<any>[] = []
    public prefix: string = "/"
    addRoute(RouteHandler) {
        this.routes
    }
    install(server: express.Application) {
        for (let route of this.routes) {
            let path = this.prefix + route.path
            let method = route.method.toLowerCase()
            server[method](path, (req, res, next) => {
                route.handleRawRequest(req, res, next)
            })
        }
    }
}
export abstract class Route<TMIME extends MIME> {
    constructor(public service: RouteService) {
        let name: string
        Object.defineProperty(this, "name", {
            get: () => {
                if (name) return name
                return `${this.method}_${this.path}`
            },
            set: (value) => {
                name = value
            }
        })
    }
    name: string
    mime: TMIME
    abstract path: string
    method: METHOD = "GET"
    params = {}
    body = {}
    query = {}
    next: Function
    req: Request
    res: Response
    handleRawRequest(req, res, next) {
        this.req = req
        this.res = res
        this.next = next
        this.body = req.body || {}
        this.params = req.params || {}
        this.query = req.query || {}
        this.handle()
    }
    end(content: any) {
        this.res.contentType(this.mime)
        this.res.end(content)
    }
    abstract handle();
}
export abstract class DynamicResourceRoute<T extends MIME> extends Route<T>{ }
export abstract class PageRoute extends Route<"text/html">{
    html(content: string) {
        this.end(content)
    }
}
export abstract class APIRoute extends Route<"application/json"> {
    success(data: any) {
        this.json({
            state: true,
            data
        })
    }
    error(err: Error) {
        this.json({
            state: false,
            error: err.name,
            message: err.message
        })
    }
    json(data) {
        this.res.json(data)
    }
}
export type MIME = "text/html" | "text/javascript" | "text/json" | "application/json" | string
export type METHOD = "GET" | "POST" | "DELETE" | "PUT"
